"
 COPYRIGHT (c) 1993 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

OrderedCollection subclass:#ChangeSet
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'System-Changes'
!

!ChangeSet class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 1993 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    used in the changes management to keep track of changes

    [author:]
        Claus Gittinger
"
! !

!ChangeSet class methodsFor:'instance creation'!

fromStream:aStream
    "build a changeSet from a stream, containing chunks.
     (i.e. either a classes sourceFile or a change-file).
     Return the changeSet."

    |changeSet chunk sawExcla lastTimeStamp tree s change|

    changeSet := self new.

    [aStream atEnd] whileFalse:[
        aStream skipSeparators.
        sawExcla := aStream peekFor:$!!.
        chunk := aStream nextChunk.
        (chunk notNil and:[chunk notEmpty]) ifTrue:[
            tree := Parser parseExpression:chunk inNameSpace:Smalltalk.
            tree == #Error ifTrue:[
                change := DoItChange new.
                change source:chunk.
                changeSet add: change.
            ] ifFalse:[    
                (tree notNil and:[tree ~~ #Error]) ifTrue:[
                    "/
                    "/ what type of chunk is this ...
                    "/
                    tree isConstant ifTrue:[
                        (s := tree evaluate) isString ifTrue:[
                            (s startsWith:'---- timestamp ') ifTrue:[
                                lastTimeStamp := s.
                            ]
                        ] ifFalse:[
                            self halt
                        ]
                    ] ifFalse:[
                        tree isMessage ifTrue:[
                            (changeSet addFromParseTree:tree andStream:aStream) ifFalse:[
                                change := DoItChange new.
                                change source:chunk.
                                changeSet add: change.
                            ]
                        ] ifFalse:[
                            self halt.
                        ]
                    ]
                ]
            ]
        ]
    ].
    ^ changeSet

    "
     ChangeSet fromStream:('changes' asFilename readStream)
     ChangeSet fromStream:('patches' asFilename readStream)
     ChangeSet fromStream:(Object source asString readStream)
    "

    "Created: / 16.2.1998 / 12:19:34 / cg"
    "Modified: / 16.2.1998 / 14:44:22 / cg"
! !

!ChangeSet class methodsFor:'ST80 compatibility'!

patches
    ^ #()

    "Created: / 27.10.1997 / 13:52:54 / cg"
! !

!ChangeSet class methodsFor:'queries'!

current
    "ST-80 compatibility: return the current changeSet"

    |p|

    (Project notNil and:[(p := Project current) notNil]) ifTrue:[
	^ p changeSet
    ]

    "
     ChangeSet current 
    "
! !

!ChangeSet methodsFor:'change management'!

addClassCategoryChange:newCategory for:aClass
    "add a classCategory change to the receiver"

    |newChange|

    newChange := ClassCategoryChange class:aClass category:newCategory.
    self add:newChange

    "Created: 3.12.1995 / 13:45:53 / cg"
    "Modified: 15.7.1996 / 09:26:52 / cg"
!

addClassDefinitionChangeFor:aClass
    "add a classDefinition change to the receiver"

    |newChange|

    newChange := ClassDefinitionChange class:aClass.
    self add:newChange

    "Modified: 15.7.1996 / 09:27:05 / cg"
!

addMethodCategoryChange:aMethod category:newCategory in:aClass
    "add a methodCategory change to the receiver"

    |newChange|

    newChange := MethodCategoryChange class:aClass
                                   selector:(aClass selectorAtMethod:aMethod)
                                   category:newCategory.
    self add:newChange

    "Modified: 15.7.1996 / 09:27:15 / cg"
!

addMethodChange:aMethod in:aClass
    "add a method change to the receiver"

    |newChange|

    newChange := MethodChange class:aClass
                           selector:(aClass selectorAtMethod:aMethod)
                             source:aMethod source
                           category:aMethod category.
    self add:newChange

    "Modified: 15.7.1996 / 09:27:21 / cg"
!

addMethodPrivacyChange:aMethod in:aClass
    "add a methodPrivacy change to the receiver"

    |newChange|

    newChange := MethodPrivacyChange class:aClass
                                   selector:(aClass selectorAtMethod:aMethod)
                                   privacy:aMethod privacy.
    self add:newChange

    "Modified: 27.8.1995 / 22:55:22 / claus"
    "Modified: 15.7.1996 / 09:27:28 / cg"
!

addPrimitiveDefinitionsChangeFor:aClass
    "add a primitiveDefinitions change to the receiver"

    |newChange|

    newChange := ClassPrimitiveDefinitionsChange class:aClass.
    self add:newChange

    "Modified: 15.7.1996 / 09:27:40 / cg"
!

addPrimitiveFunctionsChangeFor:aClass
    "add a primitiveFunctions change to the receiver"

    |newChange|

    newChange := ClassPrimitiveFunctionsChange class:aClass.
    self add:newChange

    "Modified: 15.7.1996 / 09:27:47 / cg"
!

addPrimitiveVariablesChangeFor:aClass
    "add a primitiveVariables change to the receiver"

    |newChange|

    newChange := ClassPrimitiveVariablesChange class:aClass.
    self add:newChange

    "Modified: 15.7.1996 / 09:27:55 / cg"
!

addRemoveSelectorChange:aSelector in:aClass
    "add a method-remove change to the receiver"

    |newChange|

    newChange := MethodRemoveChange 
                        class:aClass
                        selector:aSelector.
    self add:newChange

    "Modified: / 27.8.1995 / 22:55:22 / claus"
    "Modified: / 15.7.1996 / 09:27:28 / cg"
    "Created: / 16.2.1998 / 12:47:07 / cg"
! !

!ChangeSet methodsFor:'misc'!

addPatch:nameOfPatch
    "ignored for now - allows fileIn of ST-80 patch stuff .."

    ^ self
! !

!ChangeSet methodsFor:'private'!

addFromParseTree:aTree andStream:aStream
    |sel className categoryName 
     methodSource methodSelector change parser
     oldName newName priv|

    sel := aTree selector.
    (sel == #'methodsFor:'
    or:[(sel == #'ignoredMethodsFor:')]) ifTrue:[
        (sel == #'ignoredMethodsFor:') ifTrue:[
            priv := #ignored.
        ] ifFalse:[
            priv := nil
        ].
        (aTree receiver isUnaryMessage 
        and:[aTree receiver selector == #class]) ifTrue:[
            className := (aTree receiver receiver name) , ' class'.
        ] ifFalse:[
            className := (aTree receiver name).
        ].
        categoryName := (aTree arguments at:1) evaluate.

        aStream skipSeparators.
        methodSource := aStream nextChunk.
        [methodSource notEmpty] whileTrue:[
            parser := Parser
                            parseMethod:methodSource 
                            in:nil 
                            ignoreErrors:true 
                            ignoreWarnings:true.

            methodSelector := parser selector.

            change := MethodChange new.
            change 
                className:className
                selector:methodSelector
                source:methodSource
                category:categoryName
                privacy:priv.
            self add: change.

            aStream skipSeparators.
            methodSource := aStream nextChunk.
        ].
        ^ true
    ].

    sel == #'removeSelector:' ifTrue:[
        (aTree receiver isUnaryMessage 
        and:[aTree receiver selector == #class]) ifTrue:[
            className := (aTree receiver receiver name) , ' class'.
        ] ifFalse:[
            className := (aTree receiver name).
        ].
        methodSelector := (aTree arguments at:1) evaluate.
        change := MethodRemoveChange new.
        change 
            className:className
            selector:methodSelector.
        self add: change.
        ^ true
    ].

    ((sel startsWith:'subclass:')
    or:[(sel startsWith:'variableSubclass:')
    or:[(sel startsWith:'variableByteSubclass:')
    or:[(sel startsWith:'variableWordSubclass:')
    or:[(sel startsWith:'variableLongSubclass:')
    or:[(sel startsWith:'variableSignedWordSubclass:')
    or:[(sel startsWith:'variableSignedLongSubclass:')
    or:[(sel startsWith:'variableFloatSubclass:')
    or:[(sel startsWith:'variableDoubleSubclass:')
    ]]]]]]]]) ifTrue:[
        className := (aTree arguments at:1) evaluate.
        change := ClassDefinitionChange new.
        change 
            className:className;
            source:(aTree printString).
        self add: change.
        ^ true
    ].

    sel == #'renameCategory:to:' ifTrue:[
        (aTree receiver isUnaryMessage 
        and:[aTree receiver selector == #class]) ifTrue:[
            className := (aTree receiver receiver name) , ' class'.
        ] ifFalse:[
            className := (aTree receiver name).
        ].
        change := MethodCategoryRenameChange new.
        change 
            className:className;
            oldCategoryName:(aTree arguments at:1) evaluate
            newCategoryName:(aTree arguments at:2) evaluate.
        self add: change.
        ^ true
    ].

    (sel == #'category:' 
    or:[sel == #'privacy:']) ifTrue:[
        (aTree receiver isMessage
        and:[aTree receiver selector == #'compiledMethodAt:']) ifTrue:[
            (aTree receiver receiver isUnaryMessage 
            and:[aTree receiver receiver selector == #class]) ifTrue:[
                className := (aTree receiver receiver receiver name) , ' class'.
            ] ifFalse:[
                className := (aTree receiver receiver name).
            ].
            methodSelector := (aTree receiver arguments at:1) evaluate.

            sel == #'category:' ifTrue:[
                change := MethodCategoryChange new.
                change 
                    className:className
                    selector:methodSelector
                    category:(aTree arguments at:1) evaluate.
            ] ifFalse:[
                change := MethodPrivacyChange new.
                change 
                    className:className
                    selector:methodSelector
                    privacy:(aTree arguments at:1) evaluate.
            ].

            self add: change.
            ^ true
        ] ifFalse:[
            self halt
        ].
    ].

    sel == #'comment:' ifTrue:[
        (aTree receiver isUnaryMessage 
        and:[aTree receiver selector == #class]) ifTrue:[
            className := (aTree receiver receiver name) , ' class'.
        ] ifFalse:[
            className := (aTree receiver name).
        ].

        change := ClassCommentChange new.
        change 
            className:className
            comment:(aTree arguments at:1) evaluate.
        self add: change.
        ^ true
    ].

    sel == #'removeClass:' ifTrue:[
        (aTree receiver isVariable 
        and:[aTree receiver name == #Smalltalk]) ifTrue:[
            className := (aTree arguments at:1) name.

            change := ClassRemoveChange new.
            change className:className.
            self add: change.
            ^ true
        ] ifFalse:[
            self halt
        ].
    ].

    sel == #'renameClass:to:' ifTrue:[
        (aTree receiver isVariable 
        and:[aTree receiver name == #Smalltalk]) ifTrue:[
            oldName := (aTree arguments at:1) name.
            newName := (aTree arguments at:2) evaluate.

            change := ClassRenameChange new.
            change oldName:oldName newName:newName.
            self add: change.
            ^ true
        ] ifFalse:[
            self halt
        ].
    ].

    sel == #'name:' ifTrue:[
        (aTree receiver isVariable 
        and:[aTree receiver name == #Namespace]) ifTrue:[
            className := (aTree arguments at:1) evaluate.

            change := NameSpaceCreationChange new.
            change name:className.
            self add: change.
            ^ true
        ] ifFalse:[
            self halt
        ].
    ].

    ^ false

    "Created: / 16.2.1998 / 13:42:40 / cg"
    "Modified: / 16.2.1998 / 14:37:40 / cg"
! !

!ChangeSet class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libbasic3/ChangeSet.st,v 1.20 1998-02-16 13:59:37 cg Exp $'
! !
